// ----------------------------------------------------------------------------
//	File:	ofdstream.h
//	Class	ofdstream
//	Description:
//	Implements an ostream associated with a file descriptor (handle).
//	Source: Josuttis, p. 672
//	Implemented by Tom Hilinski, tom.hilinski@attbi.com, Feb03
//	Provided "as is" to the V GUI library under the same license as
//	that library.
// ----------------------------------------------------------------------------

#include <ostream>	// for std::ostream
#include <streambuf>	// for std::streambuf
#include <unistd.h>	// for write

class ofdstreambuf;

// ----------------------------------------------------------------------------
// output file descriptor stream buffer
// ----------------------------------------------------------------------------
class ofdstreambuf
	: public std::streambuf
{
  public:
  	ofdstreambuf (
  	  int useFD)
  	  : fd (useFD)
  	  {
  	  }
	~ofdstreambuf ()
	  {
	  }

  private:
	int fd;		// file descriptor

	// write one character
	virtual int overflow (
	  int c)
	  {
	    if (c != EOF)
	    {
		char z = c;
		if (write (fd, &z, 1) != 1) // failed?
		{
		  return EOF;
		}
	    }
	    return c;
	  }
	// write multiple characters
	virtual std::streamsize xsputn (
	  const char* cStr,
	  std::streamsize length)
	  {
	    return write (fd, cStr, length);
	  }
};


// ----------------------------------------------------------------------------
// output file descriptor stream
// ----------------------------------------------------------------------------
class ofdstream
	: public std::ostream
{
  public:
	ofdstream (
	  int fd)
	  : buffer (fd),
	    std::ostream (&buffer)
	  {
	  }
	~ofdstream ()
	  {
	  }

  private:
  	ofdstreambuf buffer;
};

//--- end of file ---
